@@ -1,6 +1,6 @@
 /*
  * OpenRPT report writer and rendering engine
- * Copyright (C) 2001-2012 by OpenMFG, LLC
+ * Copyright (C) 2001-2014 by OpenMFG, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 #include "metasqlhighlighter.h"
 
 #include <QColor>
+#include <QTextEdit>
 
 // #include "format.h"
 
@@ -253,7 +254,7 @@ void MetaSQLHighlighter::init()
               << "EXPLAIN"
               << "EXTERNAL"
               << "EXTRACT"
-              << "FALSE"
+              << "false"
               << "FETCH"
               << "FILTER"
               << "FINAL"
